% mrtex - Tutorial de Modelacin y Simulacin de Robots usando el
%         Robotics Toolbox de Peter Corke.
%
% Muestra funciones bsicas para modelar y simular robots.
% Los ejemplos consideran el robot "2-DOF RR-planar" de la presentacin,
% el cual consta de dos grados de libertad (hombro y codo) contenidos en
% un plano ortogonal (vertical) al de la base (horizontal).
%
% Estos ejemplos pueden modificarse con facilidad para robots ms
% complejos.  Aqu el propsito es mostrar de manera clara y sencilla, la
% potencialidad de las funciones provistas por el Robotics Toolbox.
%
% Estos ejemplos han sido realizados con la versin 8 (realease Feb. 2009) 
% del Robotics Toolbox.  El nuevo release 2011 ha sufrido varios cambios y
% una buena parte de estos ejemplos requieren modificaciones para que 
% funcionen con la nueva sintaxis y la nueva forma de declarar las
% estructuras asociadas al robot.
%
% Miguel Torres-Torriti (c) XII.2011
%
% Comentarios:
% Ejecute el cdigo por etapas segn las partes.
% 

clear all
close all

% --- Parte 1A: Crear un robot ---

% parmetros de Denavit-Hartenberg
% L = LINK([alpha A theta D joint_type])
L{1} = link([ 0	0.5 0 0 0], 'standard');
L{2} = link([ 0 0.5 0 0 0], 'standard');

% masas de los eslabones
L{1}.m = 10
L{2}.m = 10

% inercias de los eslabones
L{1}.I = [0.1 0 0; 0 L{1}.m*L{1}.A^2/12 0;0 0 L{1}.m*L{1}.A^2/12];
L{2}.I = [0.1 0 0; 0 L{2}.m*L{2}.A^2/12 0;0 0 L{2}.m*L{2}.A^2/12];

% centros de masa de los eslabones
L{1}.r = [-L{1}.A/2; 0; 0];
L{2}.r = [-L{2}.A/2; 0; 0];

% inercia de los motores
% funciones para la dinmica (ej. accel) requieren la definicion de la inercia
% de los motores, de lo contrario producen un mensaje de error:
%    Subscripted assignment dimension mismatch.
L{1}.Jm =  1e-4;
L{2}.Jm =  1e-4;

% razn de reduccin N:1
% funciones para la dinmica (ej. accel) requieren la definicion de la
% razon de reduccion, de lo contrario producen un mensaje de error:
%    Subscripted assignment dimension mismatch.
L{1}.G =   100.0;
L{2}.G =   100.0;

% friccin viscosa (referida al eje del motor)
L{1}.B =   50*1e-3;
L{2}.B =   20*1e-3;

% friccin de Coulomb (referida al eje del motor)
%L{1}.Tc = [ .1	-.1];
%L{2}.Tc = [ .1	-.1];

RRplanar = robot(L);
RRplanar.name = 'RRplanar';
RRplanar.manuf = 'mtt';

drivebot(RRplanar);

RRplanar.gravity %= [0;9.81;0];

%%break

% --- Parte 1B: Crear un robot con la orientacin correcta ---

% Note que el robot original est contenido en el plano XY.  Esta no es la
% configuracin que se deseaba crear en el primer ejemplo de la
% presentacin.
%
% Cmo hacemos que el robot est en el plano XZ y la gravedad a lo largo
% de la componente Y?

% Es necesario rotar la base entorno a X en pi/2 para dejar el eje Z0 en el
% plano horizontal.

RRplanar.base = [rotx(pi/2) zeros(3,1);zeros(1,3) 1] % usando una matrix de transformacin homognea

RRplanar.gravity = [0; 9.81; 0];
drivebot(RRplanar);


% --- Parte 1C: Generacin y despliegue de una trayectoria para el robot ---

% Dada una configuracin inicial qi de la articualciones y una final qf, se
% interpola linealmente para los instantes de tiempos en t, la trajectoria
% de las articulaciones qt y sus respectivas velocidades y aceleraciones en
% qtd y qtdd.

qi = [0 0];
qf = [pi/4 pi/2];
t = 0:0.01:5;
[qt qtd qtdd] = jtraj(qi, qf, t);

plot(RRplanar,qt,'noerase','loop')

% Otra manera de mostrar la evolucin de la trayetoria.

for q = qt', % para todos los puntos en la trayectoria
    plot(RRplanar, q');%'noerase');
    %pause(0.1)
    drawnow % si no hay pause es necesario usar drawnow
end

% --- Parte 2A: Dinmica Inversa ---

% Las funciones de la seccin 1 no requieren de la definicin de inercias
% de los motores ni sus reducciones.  Sin embargo, si estos parmetros no
% se definen las funciones de esta seccin no funcionarn, an cuando las
% inercias de los eslabones estn definidas.
%
% Un aspecto clave es la reduccin de engranajes, que debe ser 1, si no se
% conoce su valor, puesto que una reduccin 1:1 equivale a un acoplamiento
% directo sin reductor entre el motor y el eslabn.
%
% ATENCION!!!
% rne sin compilar y su versin compilada frne (fast rne) tienen
% problemas en la version release 8 Feb. 2009 de Robotics Toolbox.
%
% Observe que frne no viene precompilada.  Para compilarla siga los pasos
% explicados en la pg. 8 de la documentacin de Robotics Toolbox.
% Luego copie frne a la carpeta donde tiene los archivos .m de trabajo o
% agregue frne al path de Matlab.
%
% Note las diferencias de resultados entre rne y frne en los siguientes
% casos

tau = rne(RRplanar, [0 0], [0 0], [0 0])  % no entrega el torque correcto
                                          % para la segunda articulacin
                                          
tau = frne(RRplanar, [0 0], [0 0], [0 0]) % entrega ambos torques correctamente

% Ahora si especificamos adicionalmente el vector de gravedad:

tau = rne(RRplanar, [0 0], [0 0], [0 0], [0 9.81 0]) % nuevamente no entrega el 
                                                     % torque correcto para
                                                     % la segunda
                                                     % articulacin.

tau = rne(RRplanar, [0 0], [0 0], [0 0], [0 0 9.81]) % entrega el torque 
                                                     % correcto para la 
                                                     % segunda articulacin, 
                                                     % pero no para la primera.

                                                     
tau = frne(RRplanar, [0 0], [0 0], [0 0], [0 9.81 0]) % entrega el torque 
                                                      % correcto para ambas
                                                      % articulaciones.

tau = frne(RRplanar, [0 0], [0 0], [0 0], [0 0 9.81]) % entrega el roque 
                                                      % incorrecto para ambas
                                                      % articulaciones.

                                                      
% tau = rne(RRplanar, [qt qtd qtdd]); % entrega resultado incorrecto

tau = frne(RRplanar, qt, qtd, qtdd); % entrega resultado correcto

plot(t,tau(:,1),t,tau(:,2));
xlabel('Tiempo [s]')
ylabel('Torque de la Articulacin []')
title('Dinmica inversa para el robot 2-DOF RR-planar usando frne')
legend('\tau_1','\tau_2')
grid

% Ejercicio:
% Qu ocurrira si la trayectoria generada qt terminase en qf = [0 pi/2],
% en vez de qf = [pi/4 pi/2]?  Qu ocurre con el torque final en cada
% caso?


% --- Parte 2B: Dinmica Directa e integracin de Euler ---

% Calculo de la aceleracin qdd de cada articulacin dados q, qd, y tau,
% empleando la funcin accel, la cual implementa el mtodo de dinmica
% Directa de Walker y Orin, 1982.

q0 = zeros(1,2)+[0,pi/2];
qd0 = zeros(1,2);
tau0 = zeros(1,2);
qdd0 = accel(RRplanar, q0, qd0, tau0)

% Es posible realizar una integracin simple de las aceleraciones qdd 
% mediante el mtodo the Euler de primer orden con un periodo de muestreo
% de Ts segundos para actualizar la configuracion de las articulaciones q 
% y sus velocidades qd.
%
% En este ejemplo asumimos que el valor del torque de los motores es cero
% durante todo el periodo de simulacin.

t0 = 0;
tf = 4;
k=1;
Ts=0.01;
q(k,:)=q0;
for i=t0+Ts:Ts:tf,    
    qd0 = qd0 + qdd0'*Ts;
    q0  = q0  +  qd0*Ts;
    qdd0 = accel(RRplanar, q0, qd0, tau0);
    k = k+1;
    q(k,:)=q0;
end
plot(t0:Ts:tf,q*180/pi)
xlabel('Tiempo [s]')
ylabel('Angulo de la Articulacin []')
title('Integracin de Euler de la dinmica directa')
legend('\theta_1','\theta_2')
grid
%plot(RRplanar,q,'noerase','loop')

% --- Parte 2C: Dinmica Directa e integracin de Runge-Kutta ---

% Una manera ms exacta de obtener la trayectoria de las articulaciones es
% emplear la funcin fdyn, la cual integra las ecuaciones dinmicas del
% robot empleando el mtodo de Runge-Kutta y los resultados de la 
% dinmica directa entregados por la funcin accel.
%
% Comparando los resultados anteriores con el de este enfoque, es posible
% notar que los resultados obtenidos empleando la integracin de Euler
% empeoran considerablemente cuando aunmenta el periodo de muestreo.

q0 = zeros(1,2)+[0,pi/2];
qd0 = zeros(1,2);
tau0 = zeros(1,2);
[tA qA qdA] = fdyn(RRplanar, t0, tf, tau0, q0, qd0);
figure;
plot(tA,qA);
xlabel('Tiempo [s]')
ylabel('Angulo de la Articulacin []')
title('Integracin de Runge-Kutta usando fdyn de la dinmica directa')
legend('\theta_1','\theta_2')
grid

% Ejercicio:
% Qu pasa si la friccin viscosa de cada articulacin L{}.B es cero 
%  y las articulaciones tienen una reduccin de engranajes igual a 1?
% Qu pasa si el tiempo de muestreo Ts es 0.001 en vez de 0.01?

% break

% --- Parte 3: Ejemplo del robot controlado mediante un control PD ---
t = [t0:Ts:tf]';
q0 = [0 pi/2];
qr = [pi/2 0];
q_dmd = jtraj(q0, qr, t); % crea trayectoria
q_dmd_t = [t q_dmd];

Pgain = [200 100]; % fijar ganancias
Dgain = [-10 -10];
[tC,qC,qdC] = fdyn(nofriction(RRplanar), t0, tf, 'taufunc', q0, q0, Pgain, Dgain, q_dmd_t);
figure; hold on;
plot(t,q_dmd(:,1),'b');
plot(t,q_dmd(:,2),'Color',[0 0.5 0])
plot(tC,qC(:,1),'b-.','Linewidth',1.5)
plot(tC,qC(:,2),'-.','Linewidth',1.5,'Color',[0 0.5 0])
xlabel('Tiempo [s]')
ylabel('Angulo de la Articulacin []')
title('Control PD de las articulaciones del robot 2-DOF RR-planar')
legend('\theta_1^d','\theta_2^d','\theta_1','\theta_2')
grid

figure;
plot(RRplanar,qC,'noerase','loop')
